"""Module for building the rsync command to send files to customer inbox on caesar"""
import glob
import logging
from pathlib import Path
from typing import List


from cg.exc import CgError
from cg.meta.meta import MetaAPI
from cg.models.cg_config import CGConfig
from cg.store import models
from cg.utils import Process
from cgmodels.cg.constants import Pipeline

LOG = logging.getLogger(__name__)


class RsyncAPI(MetaAPI):
    def __init__(self, config: CGConfig):
        super().__init__(config)
        self.delivery_path: str = config.delivery_path
        self.destination_path: str = config.data_delivery.destination_path
        self.covid_destination_path: str = config.data_delivery.covid_destination_path
        self.covid_report_path: str = config.data_delivery.covid_report_path
        self._process = None

    @property
    def process(self):
        if not self._process:
            self._process = Process("rsync")
        return self._process

    def run_rsync_command(self, ticket_id: int = None, dry_run: bool = False) -> None:

        cases: List[models.Family] = self.status_db.get_cases_from_ticket(ticket_id=ticket_id).all()
        if not cases:
            LOG.warning("Could not find any cases for ticket_id %s", ticket_id)
            raise CgError()

        customer_id: str = cases[0].customer.internal_id
        rsync_destination_path: str = self.destination_path % (customer_id, ticket_id)
        delivery_source_path: str = (
            Path(self.delivery_path, customer_id, "inbox", str(ticket_id)).as_posix() + "/"
        )
        rsync_options = "-rvL"
        parameters: List[str] = [rsync_options, delivery_source_path, rsync_destination_path]
        self.process.run_command(parameters=parameters, dry_run=dry_run)

        if cases[0].data_analysis == Pipeline.SARS_COV_2:
            LOG.info("Delivering report for SARS-COV-2 analysis")
            covid_report_options: List[str] = glob.glob(
                self.covid_report_path % (str(cases[0].internal_id), ticket_id)
            )
            if not covid_report_options:
                LOG.error(
                    f"No report file could be found with path"
                    f" {self.covid_report_path % (str(cases[0].internal_id), ticket_id)}!"
                )
                return
            covid_report_path: str = covid_report_options[0]
            covid_destination_path: str = self.covid_destination_path % customer_id
            parameters: List[str] = [rsync_options, covid_report_path, covid_destination_path]
            self.process.run_command(parameters=parameters, dry_run=dry_run)



"""Commands to start MIP rare disease RNA workflow"""

import logging

import click
from cg.cli.workflow.commands import link, resolve_compression
from cg.cli.workflow.mip.base import config_case, panel, run, start, start_available
from cg.cli.workflow.mip.store import store as store_cmd
from cg.meta.workflow.mip_rna import MipRNAAnalysisAPI

LOG = logging.getLogger(__name__)


@click.group("mip-rna")
@click.pass_context
def mip_rna(context: click.Context):
    """Rare disease RNA workflow"""

    if context.invoked_subcommand is None:
        click.echo(context.get_help())
        return

    context.obj.meta_apis["analysis_api"] = MipRNAAnalysisAPI(config=context.obj)


mip_rna.add_command(config_case)
mip_rna.add_command(link)
mip_rna.add_command(panel)
mip_rna.add_command(resolve_compression)
mip_rna.add_command(run)
mip_rna.add_command(start)
mip_rna.add_command(start_available)
mip_rna.add_command(store_cmd)



import logging
import shutil
from pathlib import Path
from typing import Iterable, List, Optional

from cg.apps.cgstats.crud import create, find
from cg.apps.cgstats.stats import StatsAPI
from cg.apps.demultiplex.demultiplex_api import DemultiplexingAPI
from cg.apps.demultiplex.demux_report import create_demux_report
from cg.constants.cgstats import STATS_HEADER
from cg.exc import FlowcellError
from cg.meta.demultiplex import files
from cg.models.cg_config import CGConfig
from cg.models.cgstats.stats_sample import StatsSample
from cg.models.demultiplex.demux_results import DemuxResults
from cg.models.demultiplex.flowcell import Flowcell

LOG = logging.getLogger(__name__)


class DemuxPostProcessingAPI:
    def __init__(self, config: CGConfig):
        self.stats_api: StatsAPI = config.cg_stats_api
        self.demux_api: DemultiplexingAPI = config.demultiplex_api
        self.dry_run = False

    def set_dry_run(self, dry_run: bool) -> None:
        LOG.debug("Set dry run to %s", dry_run)
        self.dry_run = dry_run
        if dry_run:
            self.demux_api.set_dry_run(dry_run=dry_run)

    def rename_files(self, demux_results: DemuxResults) -> None:
        """Rename the files according to how we want to have it after demultiplexing is ready"""
        LOG.info("Renaming files for flowcell %s", demux_results.flowcell.flowcell_full_name)
        flowcell_id: str = demux_results.flowcell.flowcell_id
        for project_dir in demux_results.raw_projects:
            if "index" in project_dir.name:
                continue
            files.rename_project_directory(
                project_directory=project_dir, flowcell_id=flowcell_id, dry_run=self.dry_run
            )

    def add_to_cgstats(self, demux_results: DemuxResults) -> None:
        """Add the information from demultiplexing to cgstats"""
        create.create_novaseq_flowcell(manager=self.stats_api, demux_results=demux_results)

    @staticmethod
    def fetch_report_samples(flowcell_id: str, project_name: str) -> List[StatsSample]:
        samples: List[StatsSample] = find.project_sample_stats(
            flowcell=flowcell_id, project_name=project_name
        )
        LOG.info(
            "Found samples: %s for flowcell: %s, project: %s",
            ",".join(sample.sample_name for sample in samples),
            flowcell_id,
            project_name,
        )
        return samples

    @staticmethod
    def sample_to_report_line(stats_sample: StatsSample, flowcell_id: str) -> str:
        lanes: str = ",".join(str(lane) for lane in stats_sample.lanes)
        return "\t".join(
            [
                stats_sample.sample_name,
                flowcell_id,
                lanes,
                ",".join(str(unaligned.read_count) for unaligned in stats_sample.unaligned),
                str(stats_sample.read_count_sum),
                ",".join(str(unaligned.yield_mb) for unaligned in stats_sample.unaligned),
                str(stats_sample.sum_yield),
                ",".join(str(unaligned.q30_bases_pct) for unaligned in stats_sample.unaligned),
                ",".join(str(unaligned.mean_quality_score) for unaligned in stats_sample.unaligned),
            ]
        )

    @staticmethod
    def get_report_lines(stats_samples: List[StatsSample], flowcell_id: str) -> Iterable[str]:
        """Convert stats samples to format lines ready to print"""
        for stats_sample in sorted(stats_samples, key=lambda x: x.sample_name):
            yield DemuxPostProcessingAPI.sample_to_report_line(
                stats_sample=stats_sample, flowcell_id=flowcell_id
            )

    @staticmethod
    def write_report(report_path: Path, report_data: List[str]) -> None:
        LOG.info("Write demux report to %s", report_path)
        with report_path.open("w") as report_file:
            report_file.write("\t".join(STATS_HEADER) + "\n")
            for line in report_data:
                report_file.write(line + "\n")

    def get_report_data(self, flowcell_id: str, project_name: Optional[str] = None) -> List[str]:
        """Fetch the lines that are used to make a report from a flowcell"""
        LOG.info("Fetch report data for flowcell: %s project: %s", flowcell_id, project_name)
        project_samples: List[StatsSample] = self.fetch_report_samples(
            flowcell_id=flowcell_id, project_name=project_name
        )

        return [
            sample_line
            for sample_line in self.get_report_lines(
                stats_samples=project_samples, flowcell_id=flowcell_id
            )
        ]

    def create_cgstats_reports(self, demux_results: DemuxResults) -> None:
        """Create a report for every project that was demultiplexed"""
        flowcell_id: str = demux_results.flowcell.flowcell_id
        for project in demux_results.projects:
            project_name: str = project.split("_")[-1]
            report_data: List[str] = self.get_report_data(
                flowcell_id=flowcell_id, project_name=project_name
            )
            report_path: Path = demux_results.demux_dir / f"stats-{project_name}-{flowcell_id}.txt"
            self.write_report(report_path=report_path, report_data=report_data)

    @staticmethod
    def create_barcode_summary_report(demux_results: DemuxResults) -> None:
        report_content = create_demux_report(conversion_stats=demux_results.conversion_stats)
        report_path: Path = demux_results.barcode_report
        if report_path.exists():
            LOG.warning("Report path already exists!")
            return
        LOG.info("Write demux report to %s", report_path)
        with report_path.open("w") as report_file:
            report_file.write("\n".join(report_content))

    @staticmethod
    def copy_sample_sheet(demux_results: DemuxResults) -> None:
        """Copy the sample sheet from run dir to demux dir"""
        LOG.info(
            "Copy sample sheet (%s) from flowcell to demuxed result dir (%s)",
            demux_results.sample_sheet_path,
            demux_results.demux_sample_sheet_path,
        )
        shutil.copy(
            demux_results.sample_sheet_path.as_posix(),
            demux_results.demux_sample_sheet_path.as_posix(),
        )

    @staticmethod
    def create_copy_complete_file(demux_results: DemuxResults) -> None:
        """Suggestion is to deprecate this as a flag and store information in database instead"""
        LOG.info("Creating %s", demux_results.copy_complete_path)
        demux_results.copy_complete_path.touch()

    def post_process_flowcell(self, demux_results: DemuxResults) -> None:
        """Run all the necessary steps for post processing a demultiplexed flowcell

        This will
            1. rename all the necessary files and folders
            2. add the demux results to cgstats
            3. produce reports for every project
            4. generate a report with samples that have low cluster count
        """
        if demux_results.files_renamed():
            LOG.info("Files have already been renamed")
        else:
            self.rename_files(demux_results=demux_results)
        self.add_to_cgstats(demux_results=demux_results)
        self.create_cgstats_reports(demux_results=demux_results)
        self.create_barcode_summary_report(demux_results=demux_results)
        self.copy_sample_sheet(demux_results=demux_results)
        self.create_copy_complete_file(demux_results=demux_results)

    def finish_flowcell(self, flowcell_name: str, force: bool = False) -> None:
        """Go through the post processing steps for a flowcell

        Force is used to finish a flowcell even if the files are renamed already
        """
        LOG.info("Check demuxed flowcell %s", flowcell_name)
        try:
            flowcell: Flowcell = Flowcell(flowcell_path=self.demux_api.run_dir / flowcell_name)
        except FlowcellError:
            return
        if not self.demux_api.is_demultiplexing_completed(flowcell=flowcell):
            LOG.warning("Demultiplex is not ready for %s", flowcell_name)
            return
        demux_results: DemuxResults = DemuxResults(
            demux_dir=self.demux_api.out_dir / flowcell_name, flowcell=flowcell
        )
        if not demux_results.results_dir.exists():
            LOG.warning("Could not find results directory %s", demux_results.results_dir)
            LOG.info("Can not finish flowcell %s", flowcell_name)
            return
        if demux_results.files_renamed():
            LOG.warning("Flowcell is already finished!")
            if not force:
                return
            LOG.info("Post processing flowcell anyway")
        self.post_process_flowcell(demux_results=demux_results)

    def finish_all_flowcells(self) -> None:
        """Loop over all flowcells and post process those that need it"""
        demuxed_flowcells_dir: Path = self.demux_api.out_dir
        for flowcell_dir in demuxed_flowcells_dir.iterdir():
            if not flowcell_dir.is_dir():
                continue
            self.finish_flowcell(flowcell_name=flowcell_dir.name)



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import GenericRepr, Snapshot

snapshots = Snapshot()

snapshots["test_parse_config 1"] = {
    "case": "case_id",
    "email": None,
    "is_dryrun": False,
    "out_dir": "tests/fixtures/apps/mip/rna/store/",
    "priority": "low",
    "sampleinfo_path": "tests/fixtures/apps/mip/rna/store/case_qc_sample_info.yaml",
    "samples": [{"id": "sample_id", "type": "wts"}],
}

snapshots["test_parse_sampleinfo_data 1"] = {
    "case": "case_id",
    "date": GenericRepr("datetime.datetime(2019, 11, 21, 14, 31, 2)"),
    "is_finished": True,
    "version": "v7.1.4",
}



"""Tests for the upload genotypes api"""

from datetime import datetime
from pathlib import Path

import yaml
from cg.apps.mip import parse_qcmetrics
from cg.meta.upload.gt import UploadGenotypesAPI
from cg.store import models


def test_get_sample_sex():
    """Test to get the predicted sex from a MIP run using the upload genotypes API"""
    # GIVEN a UploadGenotypesAPI some qcmetrics data
    qcmetrics_file_path = "tests/fixtures/apps/mip/case_qc_metrics.yaml"
    qcmetrics_dict = yaml.safe_load(open(qcmetrics_file_path))
    qcmetrics_parsed = parse_qcmetrics.parse_qcmetrics(qcmetrics_dict)

    # WHEN fetching the predicted "gender" for the individual with id "father"
    sex = UploadGenotypesAPI.get_sample_predicted_sex(
        sample_id="ADM2", parsed_qcmetrics_data=qcmetrics_parsed
    )

    # THEN assert that the "gender" was "male"
    assert sex == "male"


def test_get_parsed_qc_metrics_data(case_qc_metrics: Path):
    """Test to get the predicted sex from a MIP run using the upload genotypes API"""
    # GIVEN a UploadGenotypesAPI and the path to a qcmetrics file with case data

    # WHEN fetching the predicted gender for the individual with id "father"

    qcmetrics_parsed = UploadGenotypesAPI.get_parsed_qc_metrics_data(case_qc_metrics)

    # THEN assert that there are three samples in the case qc metrics file
    assert len(qcmetrics_parsed) == 3


def test_get_bcf_file(upload_genotypes_api: UploadGenotypesAPI, case_id: str, timestamp: datetime):
    """Test to get the predicted sex from a MIP run using the upload genotypes API"""
    # GIVEN a UploadGenotypesAPI populated with some data in housekeeper
    hk_version = upload_genotypes_api.hk.version(case_id, timestamp)

    # WHEN fetching the gbcf file with the api
    gbcf = upload_genotypes_api.get_bcf_file(hk_version)

    # THEN assert that the file has the correct tag
    assert "snv-gbcf" in (tag.name for tag in gbcf.tags)


def test_get_data(upload_genotypes_api: UploadGenotypesAPI, analysis_obj: models.Analysis):
    """Test to get data from the UploadGenotypesAPI"""
    # GIVEN a UploadGenotypeAPI populated with some data
    # GIVEN an analysis object with a trio

    # WHEN parsing the data
    result = upload_genotypes_api.data(analysis_obj=analysis_obj)

    # THEN assert that the result looks like expected
    assert len(result["samples_sex"]) == 3



"""Mock the madeline API"""

from cg.apps.madeline.api import MadelineAPI


class MockMadelineAPI(MadelineAPI):
    """Mock the madeline api methods"""

    def __init__(self):
        """Init mock"""
        self._madeline_outpath = None

    # Mock specific functions
    def set_outpath(self, out_path: str) -> None:
        """Set the out path for mock result"""
        self._madeline_outpath = out_path

    # Mocked functions
    def run(self, family_id, samples, out_path=None):
        """Fetch version from the database."""
        return self._madeline_outpath



"""Fixtures for the orderform tests"""

import openpyxl
import pytest
from openpyxl.workbook import Workbook
from openpyxl.worksheet.worksheet import Worksheet

from cg.apps.orderform.excel_orderform_parser import ExcelOrderformParser


def get_nr_samples_excel(orderform_path: str) -> int:
    """Parse a excel orderform file and return the number of sample rows"""
    orderform_parser = ExcelOrderformParser()
    workbook: Workbook = openpyxl.load_workbook(
        filename=orderform_path, read_only=True, data_only=True
    )
    sheet_name: str = orderform_parser.get_sheet_name(workbook.sheetnames)
    orderform_sheet: Worksheet = workbook[sheet_name]
    nr_samples = 0
    current_row = "unknown"
    for row in orderform_sheet.rows:
        if row[0].value == "</SAMPLE ENTRIES>":
            # End of samples
            break
        elif row[0].value == "<SAMPLE ENTRIES>":
            # Samples start here
            current_row = "samples"
            continue
        if current_row == "samples":
            values = []
            for cell in row:
                value = str(cell.value)
                if value == "None":
                    value = ""
                values.append(value)

            # skip empty rows
            if values[0]:
                nr_samples += 1
    return nr_samples


@pytest.fixture(name="minimal_excel_sample")
def fixture_minimal_excel_sample() -> dict:
    return {
        "Sample/Name": "missingwell",
        "UDF/Data Analysis": "FLUFFY",
        "UDF/Sequencing Analysis": "RMLP05R800",
        "UDF/customer": "cust000",
    }


@pytest.fixture(name="external_order_parser")
def fixture_external_order_parser(external_orderform: str) -> ExcelOrderformParser:
    """Return a orderform parser that have parsed a external orderform in excel format"""
    order_form_parser = ExcelOrderformParser()
    order_form_parser.parse_orderform(excel_path=external_orderform)
    return order_form_parser


@pytest.fixture(name="mip_order_parser")
def fixture_mip_order_parser(mip_orderform: str) -> ExcelOrderformParser:
    """Return a orderform parser that have parsed a mip orderform in excel format"""
    order_form_parser = ExcelOrderformParser()
    order_form_parser.parse_orderform(excel_path=mip_orderform)
    return order_form_parser


@pytest.fixture(name="rml_order_parser")
def fixture_rml_order_parser(rml_orderform: str) -> ExcelOrderformParser:
    """Return a orderform parser that have parsed an orderform in excel format"""
    order_form_parser = ExcelOrderformParser()
    order_form_parser.parse_orderform(excel_path=rml_orderform)
    return order_form_parser


@pytest.fixture(name="fastq_order_parser")
def fixture_fastq_order_parser(fastq_orderform: str) -> ExcelOrderformParser:
    """Return a orderform parser that have parsed an orderform in excel format"""
    order_form_parser = ExcelOrderformParser()
    order_form_parser.parse_orderform(excel_path=fastq_orderform)
    return order_form_parser


@pytest.fixture(name="nr_samples_mip_orderform")
def fixture_nr_samples_mip_orderform(mip_orderform: str) -> int:
    """Return the number of samples in the mip orderform"""
    return get_nr_samples_excel(mip_orderform)


@pytest.fixture(name="nr_samples_rml_orderform")
def fixture_nr_samples_rml_orderform(rml_orderform: str) -> int:
    """Return the number of samples in the rml orderform"""
    return get_nr_samples_excel(rml_orderform)


@pytest.fixture(name="nr_samples_fastq_orderform")
def fixture_nr_samples_fastq_orderform(fastq_orderform: str) -> int:
    """Return the number of samples in the rml orderform"""
    return get_nr_samples_excel(fastq_orderform)


@pytest.fixture(name="mip_rna_orderform_sample")
def fixture_mip_rna_orderform_sample() -> dict:
    """Return a raw parsed mip rna sample in excel format"""
    return {
        "Sample/Name": "s1",
        "UDF/customer": "cust000",
        "UDF/Data Analysis": "MIP RNA",
        "UDF/Data Delivery": "Analysis",
        "UDF/Sequencing Analysis": "RNAPOAR025",
        "UDF/familyID": "c1",
        "UDF/Gender": "M",
        "UDF/tumor": "yes",
        "UDF/Source": "tissue (FFPE)",
        "UDF/priority": "research",
        "UDF/Process only if QC OK": "yes",
        "UDF/Volume (uL)": "1",
        "Container/Type": "96 well plate",
        "Container/Name": "plate1",
        "Sample/Well Location": "A:1",
        "UDF/Gene List": "",
        "UDF/Status": "",
        "UDF/motherID": "",
        "UDF/fatherID": "",
        "UDF/is_for_sample": "s1",
        "UDF/time_point": "0",
        "UDF/Capture Library version": "GMCKsolid",
        "UDF/Sample Buffer": 'Other (specify in "Comments")',
        "UDF/Formalin Fixation Time": "2",
        "UDF/Post Formalin Fixation Time": "3",
        "UDF/Tissue Block Size": "small",
        "UDF/Quantity": "4",
        "UDF/tumour purity": "5",
        "UDF/Comment": "other Elution buffer",
    }



from typing import List, Optional

from cg.constants import DataDelivery, Pipeline
from cg.models.orders.sample_base import OrderSample
from pydantic import Field, constr, validator


class JsonSample(OrderSample):
    case_id: str = Field(None, alias="family_name")
    concentration: Optional[str]
    concentration_sample: Optional[str]
    data_analysis: Pipeline = Pipeline.MIP_DNA
    data_delivery: DataDelivery = DataDelivery.SCOUT
    index: Optional[str]
    quantity: Optional[str]
    synopsis: Optional[List[str]]
    well_position: Optional[constr(regex=r"[A-H]:[0-9]+")]

    @validator("priority", pre=True)
    def make_lower(cls, value: str):
        return value.lower()

    @validator("well_position", pre=True)
    def convert_well(cls, value: str):
        if not value:
            return None
        if ":" in value:
            return value
        return ":".join([value[0], value[1:]])



"""Fixtures for meta/upload tests"""

from datetime import datetime
from pathlib import Path

import pytest

from cg.apps.coverage.api import ChanjoAPI
from cg.constants import Pipeline
from cg.meta.upload.coverage import UploadCoverageApi
from cg.meta.upload.gt import UploadGenotypesAPI
from cg.meta.upload.observations import UploadObservationsAPI
from cg.store import Store, models


class MockCoverage(ChanjoAPI):
    """Mock chanjo coverage api"""


class MockLoqusAPI:
    """Mock LoqusAPI class"""

    def __init__(self, analysis_type="wgs"):
        self.analysis_type = analysis_type

    @staticmethod
    def load(*args, **kwargs):
        """Mock load method"""
        _ = args
        _ = kwargs
        return dict(variants=12)

    @staticmethod
    def get_case(*args, **kwargs):
        """Mock get_case method"""
        _ = args
        _ = kwargs
        return {"case_id": "case_id", "_id": "123"}

    @staticmethod
    def get_duplicate(*args, **kwargs):
        """Mock get_duplicate method"""
        _ = args
        _ = kwargs
        return {"case_id": "case_id"}


@pytest.fixture(name="upload_genotypes_hk_bundle")
def fixture_upload_genotypes_hk_bundle(
    case_id: str, timestamp, case_qc_metrics: Path, bcf_file: Path
) -> dict:
    """Returns a dictionary in hk format with files used in upload gt process"""
    data = {
        "name": case_id,
        "created": timestamp,
        "expires": timestamp,
        "files": [
            {"path": str(case_qc_metrics), "archive": False, "tags": ["qcmetrics"]},
            {"path": str(bcf_file), "archive": False, "tags": ["snv-gbcf"]},
        ],
    }
    return data


@pytest.fixture(name="analysis_obj")
def fixture_analysis_obj(
    analysis_store_trio: Store, case_id: str, timestamp: datetime, helpers
) -> models.Analysis:
    """Return a analysis object with a trio"""
    case_obj = analysis_store_trio.family(case_id)
    helpers.add_analysis(store=analysis_store_trio, case=case_obj, started_at=timestamp)
    return analysis_store_trio.family(case_id).analyses[0]


@pytest.fixture(name="upload_genotypes_api")
def fixture_upload_genotypes_api(
    real_housekeeper_api, genotype_api, upload_genotypes_hk_bundle, helpers
) -> UploadGenotypesAPI:
    """Create a upload genotypes api"""
    helpers.ensure_hk_bundle(real_housekeeper_api, upload_genotypes_hk_bundle, include=True)
    _api = UploadGenotypesAPI(
        hk_api=real_housekeeper_api,
        gt_api=genotype_api,
    )

    return _api


@pytest.fixture(scope="function")
def upload_observations_api(analysis_store, populated_housekeeper_api):
    """Create mocked UploadObservationsAPI object"""

    loqus_mock = MockLoqusAPI()

    _api = UploadObservationsAPI(
        status_api=analysis_store,
        hk_api=populated_housekeeper_api,
        loqus_api=loqus_mock,
    )

    yield _api


@pytest.fixture(scope="function")
def upload_observations_api_wes(analysis_store, populated_housekeeper_api):
    """Create mocked UploadObservationsAPI object"""

    loqus_mock = MockLoqusAPI(analysis_type="wes")

    _api = UploadObservationsAPI(
        status_api=analysis_store,
        hk_api=populated_housekeeper_api,
        loqus_api=loqus_mock,
    )

    yield _api


@pytest.yield_fixture(scope="function")
def coverage_upload_api(chanjo_config_dict, populated_housekeeper_api):
    """Fixture for coverage upload API"""
    hk_api = populated_housekeeper_api
    status_api = None
    coverage_api = MockCoverage(chanjo_config_dict)
    _api = UploadCoverageApi(status_api=status_api, hk_api=hk_api, chanjo_api=coverage_api)
    return _api


@pytest.fixture(scope="function")
def analysis(analysis_store, case_id, timestamp):
    """Fixture to mock an analysis"""
    _analysis = analysis_store.add_analysis(pipeline=Pipeline.BALSAMIC, version="version")
    _analysis.family = analysis_store.family(case_id)
    _analysis.config_path = "dummy_path"
    _analysis.completed_at = timestamp
    yield _analysis



"""Code for uploading genotype data via CLI"""
import logging
from typing import List

import click

from cg.meta.upload.gisaid import GisaidAPI
from cg.meta.upload.gisaid.models import UpploadFiles, GisaidSample
from cg.models.cg_config import CGConfig
from cg.store import Store
from cg.store.models import Family

LOG = logging.getLogger(__name__)


@click.command()
@click.argument("family_id", required=True)
@click.pass_obj
def gisaid(context: CGConfig, family_id: str):
    """Upload mutant analysis data to GISAID."""

    LOG.info("----------------- GISAID -------------------")

    gisaid_api = GisaidAPI(config=context)
    gsaid_samples: List[GisaidSample] = gisaid_api.get_gsaid_samples(family_id=family_id)
    files: UpploadFiles = UpploadFiles(
        csv_file=gisaid_api.build_gisaid_csv(
            gsaid_samples=gsaid_samples, file_name=f"{family_id}.csv"
        ),
        fasta_file=gisaid_api.build_gisaid_fasta(
            gsaid_samples=gsaid_samples, file_name=f"{family_id}.fasta", family_id=family_id
        ),
    )
    try:
        gisaid_api.auth()
    except:
        raise
    if files:
        gisaid_api.upload(**dict(files))
        files.csv_file.unlink()
        files.fasta_file.unlink()



import json
import logging
import tempfile
from datetime import datetime
from json.decoder import JSONDecodeError
from pathlib import Path
from typing import Dict, List, Optional

from cg.utils.date import get_date
from cgmodels.crunchy.metadata import CrunchyFile, CrunchyMetadata

LOG = logging.getLogger(__name__)


# Methods to get file information
def get_log_dir(file_path: Path) -> Path:
    """Return the path to where logs should be stored"""
    return file_path.parent


def get_fastq_to_spring_sbatch_path(log_dir: Path, run_name: str = None) -> Path:
    """Return the path to where compression sbatch should be printed"""
    return log_dir / "_".join([run_name, "compress_fastq.sh"])


def get_spring_to_fastq_sbatch_path(log_dir: Path, run_name: str = None) -> Path:
    """Return the path to where decompression sbatch should be printed"""
    return log_dir / "_".join([run_name, "decompress_spring.sh"])


def get_tmp_dir(prefix: str, suffix: str, base: str = None) -> str:
    """Create a temporary directory and return the path to it"""

    with tempfile.TemporaryDirectory(prefix=prefix, suffix=suffix, dir=base) as dir_name:
        tmp_dir_path = dir_name

    LOG.info("Created temporary dir %s", tmp_dir_path)
    return tmp_dir_path


def get_crunchy_metadata(metadata_path: Path) -> CrunchyMetadata:
    """Validate content of metadata file and return mapped content"""
    LOG.info("Fetch SPRING metadata from %s", metadata_path)
    with open(metadata_path, "r") as infile:
        try:
            content: List[Dict[str, str]] = json.load(infile)
        except JSONDecodeError:
            message = "No content in SPRING metadata file"
            LOG.warning(message)
            raise SyntaxError(message)
    metadata = CrunchyMetadata(files=content)

    if len(metadata.files) != 3:
        LOG.warning("Wrong number of files in SPRING metadata file: %s", metadata_path)
        LOG.info("Found %s files, should always be 3 files", len(metadata.files))
        raise SyntaxError

    return metadata


def get_file_updated_at(crunchy_metadata: CrunchyMetadata) -> Optional[datetime.date]:
    """Check if a SPRING metadata file has been updated and return the date when updated"""
    return crunchy_metadata.files[0].updated


def get_spring_archive_files(crunchy_metadata: CrunchyMetadata) -> Dict[str, CrunchyFile]:
    """Map the files in SPRING metadata to a dictionary

    Returns: {
                "fastq_first" : {file_info},
                "fastq_second" : {file_info},
                "spring" : {file_info},
              }
    """
    names_map = {"first_read": "fastq_first", "second_read": "fastq_second", "spring": "spring"}
    archive_files = {}
    for file_info in crunchy_metadata.files:
        file_name = names_map[file_info.file]
        archive_files[file_name] = file_info
    return archive_files


def update_metadata_date(spring_metadata_path: Path) -> None:
    """Update date in the SPRING metadata file to today date"""

    now: datetime = get_date()
    spring_metadata: CrunchyMetadata = get_crunchy_metadata(spring_metadata_path)
    LOG.info("Adding today date to SPRING metadata file")
    for file_info in spring_metadata.files:
        file_info.updated = now.date()

    content: dict = json.loads(spring_metadata.json(exclude_none=True))
    with open(spring_metadata_path, "w") as outfile:
        outfile.write(json.dumps(content["files"]))



"""Mock the crunchy API"""

import logging
from pathlib import Path

from cg.apps.crunchy.crunchy import CrunchyAPI
from cg.models import CompressionData

LOG = logging.getLogger(__name__)


class MockCrunchyAPI(CrunchyAPI):
    """
    Mock for the crunchy API
    """

    def __init__(self, config: dict = None):

        self.config = config
        self.dry_run = False
        self._compression_pending_files = {}
        self._compression_possible_files = {}

        self._compression_pending = False
        self._compression_possible = True
        self._compression_done_all = False
        self._spring_compression_done = False

        self._nr_fastq_compressions = 0

    def set_dry_run(self, dry_run: bool):
        """Update dry run"""
        self.dry_run = dry_run

    # Mock specific methods
    def set_compression_pending(self, file_path: Path):
        """Set that the compression for a file is pending"""
        self._compression_pending_files[file_path] = True

    def set_compression_pending_all(
        self,
    ):
        """Set that the compression for a file is pending"""
        self._compression_pending = True

    def set_compression_not_possible(self, file_path: Path):
        """Set the variable so that bam/cram compression is not possible"""
        print("Setting compression to not possible for %s" % file_path)
        self._compression_possible_files[file_path] = False

    def set_spring_compression_done(self):
        """Set if spring compression should be in done state"""
        self._compression_done_all = True

    def nr_fastq_compressions(self):
        """Return the number of times that fastq compression was called"""
        return self._nr_fastq_compressions

    # Mocked methods
    def fastq_to_spring(self, compression_obj: CompressionData, sample_id: str = ""):
        """
        Compress FASTQ files into SPRING by sending to sbatch SLURM
        """
        self._nr_fastq_compressions += 1
        self.set_compression_pending_all()

    def spring_to_fastq(self, compression_obj: CompressionData, sample_id: str = ""):
        """Mock method that compress spring to fastq"""
        self._nr_fastq_compressions += 1
        self.set_compression_pending_all()

    def update_metadata_date(self, spring_metadata_path):
        print(f"Updates {spring_metadata_path}")

    def is_fastq_compression_possible(self, compression_obj: CompressionData) -> bool:
        """Check if it compression/decompression is possible"""
        if self._compression_pending:
            print("Compression pending")
            return False

        if self._compression_pending_files.get(compression_obj.fastq_first):
            print(f"Compression pending for file {compression_obj.fastq_first}")
            return False

        if self._compression_done_all:
            print(f"Compression done")
            return False

        compression_possible = self._compression_possible_files.get(
            compression_obj.fastq_first, self._compression_possible
        )
        print(f"Compression possible {compression_possible}")
        return compression_possible

    def is_fastq_compression_done(self, compression_obj: CompressionData) -> bool:
        """Check if spring compression if finished"""
        return self._compression_done_all

    def is_compression_pending(self, compression_obj: CompressionData) -> bool:
        """Check if compression has started, but not yet finished"""
        return self._compression_pending



from cg.apps.cgstats.crud import create, find
from cg.apps.cgstats.stats import StatsAPI
from cg.models.demultiplex.demux_results import DemuxResults


def test_create_novaseq_flowcell(stats_api: StatsAPI, demux_results: DemuxResults):
    # GIVEN a setup with an existing sample sheet with information
    assert demux_results.flowcell.sample_sheet_path.read_text()
    # GIVEN that the flowcell does not exist in the database
    assert not find.get_flowcell_id(flowcell_name=demux_results.flowcell.flowcell_id)

    # WHEN creating the novaseq flowcell
    create.create_novaseq_flowcell(manager=stats_api, demux_results=demux_results)

    # THEN assert that the flowcell was created
    assert find.get_flowcell_id(flowcell_name=demux_results.flowcell.flowcell_id)



"""Tests for the samples to deliver api"""

import datetime as dt


def test_list_samples_to_deliver(base_store, helpers):
    """Test to fetch samples ready for delivery"""
    store = base_store
    # GIVEN a populated store without samples
    assert store.samples().count() == 0
    # GIVEN inserting a sample that should be delivered
    helpers.add_sample(store, sequenced_at=dt.datetime.now())
    assert store.samples().count() == 1

    # WHEN asking for samples to deliver
    samples_to_deliver = store.samples_to_deliver()
    # THEN it should return the sample which is ready to deliver
    assert samples_to_deliver.count() == 1
    assert isinstance(samples_to_deliver.first().sequenced_at, dt.datetime)


def test_list_samples_to_deliver_multiple_samples(base_store, helpers):
    """Test to fetch samples ready for delivery and avoid the ones that are not"""
    store = base_store
    # GIVEN a populated store with two samples where one is scheduled for delivery
    helpers.add_sample(store, sequenced_at=dt.datetime.now())
    helpers.add_sample(
        store,
        sample_id="delivered",
        sequenced_at=dt.datetime.now(),
        delivered_at=dt.datetime.now(),
    )
    assert store.samples().count() == 2

    # WHEN asking for samples to deliver
    samples_to_deliver = store.samples_to_deliver()
    # THEN it should return the sample which is ready to deliver
    assert samples_to_deliver.count() == 1
    assert isinstance(samples_to_deliver.first().sequenced_at, dt.datetime)



"""Click commands to store mip analyses"""
import logging
from pathlib import Path

import click
from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.apps.lims import LimsAPI
from cg.apps.scout.scoutapi import ScoutAPI
from cg.apps.tb import TrailblazerAPI
from cg.apps.tb.models import TrailblazerAnalysis
from cg.constants import EXIT_FAIL, EXIT_SUCCESS, Pipeline
from cg.exc import (
    AnalysisDuplicationError,
    AnalysisNotFinishedError,
    BundleAlreadyAddedError,
    MandatoryFilesMissing,
    PipelineUnknownError,
)
from cg.meta.store.base import gather_files_and_bundle_in_housekeeper
from cg.meta.workflow.mip import MipAnalysisAPI
from cg.meta.workflow.mip_dna import MipDNAAnalysisAPI
from cg.models.cg_config import CGConfig
from cg.store import Store

LOG = logging.getLogger(__name__)


@click.group()
@click.pass_obj
def store(context: CGConfig):
    """Store results from MIP in housekeeper"""

    context.meta_apis["analysis_api"] = MipDNAAnalysisAPI(context)


@store.command()
@click.argument("config-stream", type=click.File("r"), required=False)
@click.pass_obj
def analysis(context: CGConfig, config_stream):
    """Store a finished analysis in Housekeeper"""

    analysis_api: MipDNAAnalysisAPI = context.meta_apis["analysis_api"]

    exit_code = EXIT_SUCCESS
    if not config_stream:
        LOG.error("Provide a config file.")
        raise click.Abort

    try:
        new_analysis = gather_files_and_bundle_in_housekeeper(
            config_stream=config_stream,
            hk_api=analysis_api.housekeeper_api,
            status=analysis_api.status_db,
            workflow=Pipeline.MIP_DNA,
        )
        analysis_api.status_db.add_commit(new_analysis)
    except (
        AnalysisNotFinishedError,
        AnalysisDuplicationError,
        BundleAlreadyAddedError,
        PipelineUnknownError,
        MandatoryFilesMissing,
    ) as error:
        LOG.error(error.message)
        exit_code = EXIT_FAIL
    except FileNotFoundError as error:
        LOG.error(f"Missing file: {error}")
        exit_code = EXIT_FAIL
    if exit_code:
        raise click.Abort

    LOG.info("Included files in Housekeeper")


@store.command()
@click.pass_context
def completed(context: click.Context):
    """Store all completed analyses"""

    analysis_api: MipAnalysisAPI = context.obj.meta_apis["analysis_api"]

    exit_code = EXIT_SUCCESS
    for case_obj in analysis_api.status_db.cases_to_store(pipeline=Pipeline.MIP_DNA):
        try:
            analysis_obj: TrailblazerAnalysis = analysis_api.trailblazer_api.get_latest_analysis(
                case_id=case_obj.internal_id
            )
            if analysis_obj.status != "completed":
                continue
            LOG.info(f"Storing case: {analysis_obj.family}")
            with Path(
                analysis_api.get_case_config_path(case_id=case_obj.internal_id)
            ).open() as config_stream:

                context.invoke(analysis, config_stream=config_stream)
        except (Exception, click.Abort) as error:
            LOG.error(f"Case storage failed: {case_obj.internal_id}, {error}")
            exit_code = EXIT_FAIL
    if exit_code:
        raise click.Abort



"""
    Cg Exceptions
"""


class CgError(Exception):

    """
    Base exception for the package
    """

    def __init__(self, message: str = ""):
        super(CgError, self).__init__()
        self.message = message


class FlowcellError(CgError):
    """Raised when there is a problem with demultiplexing a flowcell"""


class DecompressionNeededError(CgError):
    """Raised when decompression still needed to start analysis"""


class FlowcellsNeededError(CgError):
    """Raised when fetching flowcells still needed to start analysis"""


class AnalysisNotFinishedError(CgError):
    """
    Exception raised when an adding a MIP analysis to Housekeeper, but the analysis is not
    finished in MIP, as indicated in the qc sample info file.

    """


class AnalysisDuplicationError(CgError):
    """
    Error related to trying to create analysis object that already exists in status-db.
    """


class AnalysisUploadError(CgError):
    """
    Error related to trying to upload analysis data.
    """


class CgDataError(CgError):
    """
    Error related to missing/incomplete data in Status DB
    """


class LimsDataError(CgError):
    """
    Error related to missing/incomplete data in LIMS
    """


class DuplicateRecordError(CgError):
    """
    Exception related to duplicate records in LoqusDB.
    """


class DuplicateSampleError(CgError):
    """
    Exception raised when sample duplicate is found in loqusdb
    """


class CaseNotFoundError(CgError):
    """
    Exception raised when a case is not found in loqusdb
    """


class OrderFormError(CgError):
    """
    Exception related to the order form
    """


class OrderError(CgError):
    """
    Exception related to orders
    """


class TicketCreationError(CgError):
    """
    Exception related to ticket creation
    """


class BalsamicStartError(CgError):
    """
    Exception raised when Balsamic fails to start
    """


class BundleAlreadyAddedError(CgError):
    """
    Exception raised when a bundle has already been added to Housekeeper
    """


class PipelineUnknownError(CgError):
    """
    Exception raised when a sample in a case has no data analysis type
    """


class DeliveryReportError(CgError):
    """
    Exception related to delivery report creation
    """


class ValidationError(CgError):
    """
    Exception related to delivery report validation
    """


class MandatoryFilesMissing(CgError):
    """
    Exception raised when mandatory files are missing from the deliverables when storing an
    analysis in Housekeeper.
    """


class HousekeeperVersionMissingError(CgError):
    """
    Exception raised when family version is missing in Housekeeper.
    """


class HousekeeperFileMissingError(CgError):
    """
    Exception raised when a file is missing in Housekeeper.
    """

    def __init__(self, message, errors=None):
        self.message = message
        self.errors = errors


class FastaSequenceMissingError(CgError):
    """
    Exception raised when expected sequence in fasta file is missing.
    """


class StoreError(CgError):
    """
    Exception related to storing an analysis
    """


class MipStartError(CgError):
    """
    Exception raised when MIP fails to start a run
    """


class PedigreeConfigError(CgError):
    """
    Raised when MIP pedigree config validation fails
    """

    def __init__(self, message, errors=None):
        self.message = message
        self.errors = errors


class TrailblazerAPIHTTPError(CgError):
    """Raised when Trailblazer REST API response code is not 200"""


class TrailblazerMissingAnalysisError(CgError):
    """Raised when Trailblazer REST API response code is not 200"""


class MultipleFamilyLinksError(CgError):
    """Raised when only one family was expected but more than one was found"""


class FamilyLinkMissingError(CgError):
    """Raised when faimly link missing for a sample"""



"""Commands to mimic the behaviour from cgstats.

This is a way to interact with cgstats manually. In automated production these commands will not be run.
"""
import logging
from pathlib import Path
from typing import List, Optional

import click
from cg.apps.cgstats.crud.create import create_novaseq_flowcell
from cg.apps.cgstats.stats import StatsAPI
from cg.apps.demultiplex.demultiplex_api import DemultiplexingAPI
from cg.constants.cgstats import STATS_HEADER
from cg.exc import FlowcellError
from cg.meta.demultiplex.demux_post_processing import DemuxPostProcessingAPI
from cg.models.cg_config import CGConfig
from cg.models.demultiplex.demux_results import DemuxResults
from cg.models.demultiplex.flowcell import Flowcell

LOG = logging.getLogger(__name__)


@click.command(name="add")
@click.argument("flowcell-id")
@click.pass_obj
def add_flowcell_cmd(context: CGConfig, flowcell_id: str):
    """Add a flowcell to the cgstats database"""
    stats_api: StatsAPI = context.cg_stats_api
    demultiplex_api: DemultiplexingAPI = context.demultiplex_api
    flowcell_run_path: Path = demultiplex_api.run_dir / flowcell_id
    if not flowcell_run_path.exists():
        LOG.warning("Could not find flowcell path %s", flowcell_run_path)
        raise click.Abort
    demux_results_path: Path = demultiplex_api.out_dir / flowcell_id
    if not demux_results_path.exists():
        LOG.warning("Could not find demultiplex result path %s", demux_results_path)
        raise click.Abort
    try:
        flowcell: Flowcell = Flowcell(flowcell_path=flowcell_run_path)
    except FlowcellError:
        raise click.Abort
    demux_results: DemuxResults = DemuxResults(demux_dir=demux_results_path, flowcell=flowcell)
    create_novaseq_flowcell(manager=stats_api, demux_results=demux_results)


@click.command(name="select")
@click.argument("flowcell-id")
@click.option("--project", help="Name of project to fetch data for")
@click.pass_obj
def select_project_cmd(context: CGConfig, flowcell_id: str, project: Optional[str]):
    """Select a flowcell to fetch statistics from"""
    # Need to instantiate API
    post_processing_api = DemuxPostProcessingAPI(config=context)
    report_content: List[str] = post_processing_api.get_report_data(
        flowcell_id=flowcell_id, project_name=project
    )

    click.echo("\t".join(STATS_HEADER))
    if not report_content:
        LOG.warning("Could not find any samples for flowcell %s, project %s", flowcell_id, project)
        return
    for report_line in report_content:
        click.echo(report_line)



import datetime
import logging
from pathlib import Path
from typing import List, Optional

from cg.exc import FlowcellError
from cg.models.demultiplex.run_parameters import RunParameters
from cgmodels.demultiplex.sample_sheet import SampleSheet, get_sample_sheet_from_file
from cgmodels.exceptions import SampleSheetError
from pydantic import ValidationError
from typing_extensions import Literal

LOG = logging.getLogger(__name__)


class Flowcell:
    """Class to collect information about flowcell directories and there particular files"""

    def __init__(self, flowcell_path: Path):
        LOG.debug("Instantiating Flowcell with path %s", flowcell_path)
        self.path = flowcell_path
        self._run_parameters: Optional[RunParameters] = None
        self.run_date: datetime.datetime = datetime.datetime.now()
        self.machine_name: str = ""
        self.machine_number: int = 0
        # Base name is flowcell-id + flowcell position
        self.base_name: str = ""
        self.flowcell_id: str = ""
        self.flowcell_position: Literal["A", "B"] = "A"
        self.parse_flowcell_name()

    def parse_flowcell_name(self):
        """Parse relevant information from flowcell name

        This will assume that the flowcell naming convention is used. If not we skip the flowcell.
        Convention is: <date>_<machine>_<run_numbers>_<A|B><flowcell_id>
        Example: 201203_A00689_0200_AHVKJCDRXX
        """
        split_name: List[str] = self.path.name.split("_")
        if len(split_name) != 4:
            message = f"Flowcell {self.path.name} does not follow the flowcell naming convention"
            LOG.warning(message)
            raise FlowcellError(message)
        self.run_date = datetime.datetime.strptime(split_name[0], "%y%m%d")
        self.machine_name = split_name[1]
        self.machine_number = int(split_name[2])
        base_name: str = split_name[-1]
        self.base_name = base_name
        LOG.debug("Set flowcell id to %s", self.flowcell_id)
        self.flowcell_id = base_name[1:]
        self.flowcell_position = base_name[0]

    @property
    def flowcell_full_name(self) -> str:
        return self.path.name

    @property
    def sample_sheet_path(self) -> Path:
        return Path(self.path, "SampleSheet.csv")

    @property
    def run_parameters_path(self) -> Path:
        return self.path / "RunParameters.xml"

    @property
    def run_parameters_object(self) -> RunParameters:
        if not self.run_parameters_path.exists():
            message = f"Could not find run parameters file {self.run_parameters_path}"
            LOG.warning(message)
            raise FileNotFoundError(message)
        if not self._run_parameters:
            self._run_parameters = RunParameters(run_parameters_path=self.run_parameters_path)
        return self._run_parameters

    @property
    def rta_complete_path(self) -> Path:
        return Path(self.path, "RTAComplete.txt")

    @property
    def copy_complete_path(self) -> Path:
        return Path(self.path, "CopyComplete.txt")

    @property
    def demultiplexing_started_path(self) -> Path:
        return Path(self.path, "demuxstarted.txt")

    @property
    def trailblazer_config_path(self) -> Path:
        return Path(self.path, "slurm_job_ids.yaml")

    def is_demultiplexing_started(self) -> bool:
        """Create the path to where the demuliplexed result should be produced"""
        return self.demultiplexing_started_path.exists()

    def sample_sheet_exists(self) -> bool:
        """Check if sample sheet exists"""
        LOG.info("Check if sample sheet exists")
        return self.sample_sheet_path.exists()

    def validate_sample_sheet(self) -> bool:
        """Validate if sample sheet is on correct format"""
        try:
            get_sample_sheet_from_file(infile=self.sample_sheet_path, sheet_type="S4")
        except (SampleSheetError, ValidationError) as error:
            LOG.warning("Invalid sample sheet")
            LOG.warning(error)
            return False
        return True

    def get_sample_sheet(self) -> SampleSheet:
        return get_sample_sheet_from_file(infile=self.sample_sheet_path, sheet_type="S4")

    def is_sequencing_done(self) -> bool:
        """Check if sequencing is done

        This is indicated by that the file RTAComplete.txt exists
        """
        LOG.info("Check if sequencing is done")
        return self.rta_complete_path.exists()

    def is_copy_completed(self) -> bool:
        """Check if copy of flowcell is done

        This is indicated by that the file CopyComplete.txt exists
        """
        LOG.info("Check if copy of data from sequence instrument is ready")
        return self.copy_complete_path.exists()

    def is_flowcell_ready(self) -> bool:
        """Check if a flowcell is ready for demultiplexing

        A flowcell is ready if the two files RTAComplete.txt and CopyComplete.txt exists in the flowcell directory
        """
        LOG.info("Check if flowcell is ready for demultiplexing")
        if not self.is_sequencing_done():
            LOG.info("Sequencing is not completed for flowcell %s", self.flowcell_id)
            return False
        LOG.debug("Sequence is done for flowcell %s", self.flowcell_id)
        if not self.is_copy_completed():
            LOG.info("Copy of sequence data is not ready for flowcell %s", self.flowcell_id)
            return False
        LOG.debug("All data has been transferred for flowcell %s", self.flowcell_id)
        LOG.info("Flowcell %s is ready for demultiplexing", self.flowcell_id)
        return True

    def __str__(self):
        return f"Flowcell(path={self.path},run_parameters_path={self.run_parameters_path})"



"""Test module cg.cli.upload.nipt"""

import logging

from click.testing import CliRunner

from cg.cli.upload.nipt import nipt_upload_all, nipt_upload_case
from cg.models.cg_config import CGConfig


def test_nipt_upload_case(upload_context: CGConfig, cli_runner: CliRunner, caplog):
    """Tests CLI command to upload a single case"""

    caplog.set_level(logging.DEBUG)
    # GIVEN a specified NIPT case
    case_id = "angrybird"

    # WHEN adding a result file of a specified NIPT case
    # result = cli_runner.invoke(nipt_upload_case, ["--dry-run", case_id], obj=upload_context)
    result = cli_runner.invoke(nipt_upload_case, [case_id], obj=upload_context)

    # THEN the nipt upload should start and exit without errors
    assert "*** NIPT UPLOAD START ***" in caplog.text
    assert result.exit_code == 0


def test_nipt_upload_case_dry(upload_context: CGConfig, cli_runner: CliRunner, caplog):
    """Tests CLI command to upload a single case"""

    caplog.set_level(logging.DEBUG)
    # GIVEN a specified NIPT case
    case_id = "angrybird"

    # WHEN adding a result file of a specified NIPT case (dry run)
    result = cli_runner.invoke(nipt_upload_case, ["--dry-run", case_id], obj=upload_context)

    # THEN the nipt upload should start and exit without errors
    assert "*** NIPT UPLOAD START ***" in caplog.text
    assert result.exit_code == 0


def test_nipt_upload_all(upload_context: CGConfig, cli_runner: CliRunner, caplog):
    """Tests CLI command to upload a single case"""

    caplog.set_level(logging.DEBUG)
    # GIVEN adding all available NIPT case results

    # WHEN adding a result file of a all available NIPT cases
    result = cli_runner.invoke(nipt_upload_all, obj=upload_context)

    # THEN the NIPT upload should start and exit without errors
    assert "*** UPLOAD ALL AVAILABLE NIPT RESULTS ***" in caplog.text
    assert result.exit_code == 0


def test_nipt_upload_all_dry(upload_context: CGConfig, cli_runner: CliRunner, caplog):
    """Tests CLI command to upload a single case"""

    caplog.set_level(logging.DEBUG)
    # GIVEN adding all available NIPT case results

    # WHEN adding a result file of a all available NIPT cases (dry run)
    result = cli_runner.invoke(nipt_upload_all, ["--dry-run"], obj=upload_context)

    # THEN the NIPT upload should start and exit without errors
    assert "*** UPLOAD ALL AVAILABLE NIPT RESULTS ***" in caplog.text
    assert result.exit_code == 0



"""Interactions with the gisaid cli upload"""

import logging
from pathlib import Path
from typing import List, Optional

from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.apps.lims import LimsAPI
from cg.models.cg_config import CGConfig
from cg.store import models, Store
from cg.utils import Process
from .constants import HEADERS
from .models import GisaidSample, FastaFile
from housekeeper.store import models as hk_models
import csv

from cg.exc import HousekeeperVersionMissingError, FastaSequenceMissingError

LOG = logging.getLogger(__name__)


class GisaidAPI:
    """Interface with Gisaid cli uppload"""

    def __init__(self, config: CGConfig):
        self.housekeeper_api: HousekeeperAPI = config.housekeeper_api
        self.lims_api: LimsAPI = config.lims_api
        self.status_db: Store = config.status_db
        self.gisaid_submitter: str = config.gisaid.submitter
        self.gisaid_binary: str = config.gisaid.binary_path
        self.process = Process(binary=self.gisaid_binary)

    def get_new_header(self, samples: List[GisaidSample], old_header: str) -> str:
        for sample in samples:
            sample_id = sample.covv_subm_sample_id
            if old_header.find(sample_id) != -1:
                return f">{sample.covv_virus_name}\n"
        raise FastaSequenceMissingError

    def get_gisaid_fasta(
        self, gsaid_samples: List[GisaidSample], family_id: str
    ) -> List[FastaFile]:
        """Fetch a fasta files form house keeper for batch upload to gisaid"""

        hk_version: hk_models.Version = self.housekeeper_api.last_version(bundle=family_id)
        if not hk_version:
            LOG.info("Family ID: %s not found in housekeeper", family_id)
            raise HousekeeperVersionMissingError
        fasta_file: str = self.housekeeper_api.files(
            version=hk_version.id, tags=["consensus"]
        ).first()
        gisaid_delivery_fasta = []
        with open("/Users/maya.brandi/opt/395698.consensus.fa") as handle:
            fasta_lines = handle.readlines()
            for line in fasta_lines:
                if line[0] == ">":
                    gisaid_delivery_fasta.append(
                        self.get_new_header(old_header=line, samples=gsaid_samples)
                    )
                else:
                    gisaid_delivery_fasta.append(line)

        return gisaid_delivery_fasta

    def build_gisaid_fasta(
        self, gsaid_samples: List[GisaidSample], file_name: str, family_id: str
    ) -> Path:
        """Concatenates a list of consensus fastq objects"""

        file: Path = Path(file_name)
        fasta_lines: List[str] = self.get_gisaid_fasta(
            gsaid_samples=gsaid_samples, family_id=family_id
        )
        with open(file, "w") as write_file_obj:
            write_file_obj.writelines(fasta_lines)
        return file

    def get_sample_row(self, gisaid_sample: GisaidSample) -> List[str]:
        """Build row for a sample in the batch upload csv."""

        sample_dict = gisaid_sample.dict()
        return [sample_dict.get(header, "") for header in HEADERS]

    def get_sample_rows(self, gsaid_samples: List[GisaidSample]) -> List[List[str]]:
        """Build sample row list for gisad csv file"""

        sample_rows = []
        for sample in gsaid_samples:
            sample_row: List[str] = self.get_sample_row(gisaid_sample=sample)
            sample_rows.append(sample_row)
        return sample_rows

    def build_gisaid_csv(self, gsaid_samples: List[GisaidSample], file_name: str) -> Path:
        """Build batch upload csv."""

        file: Path = Path(file_name)
        with open(file_name, "w", newline="\n") as gisaid_csv:
            wr = csv.writer(gisaid_csv, delimiter=",")
            wr.writerow(HEADERS)
            sample_rows: List[List[str]] = self.get_sample_rows(gsaid_samples=gsaid_samples)
            wr.writerows(sample_rows)

        return file

    def get_gsaid_samples(self, family_id: str) -> List[GisaidSample]:
        """Get list of Gisaid sample objects."""

        samples: List[models.Sample] = self.status_db.get_sequenced_samples(family_id=family_id)
        gisaid_samples = []
        for sample in samples:
            sample_id: str = sample.internal_id
            gisaid_sample = GisaidSample(
                family_id=family_id,
                cg_lims_id=sample_id,
                covv_subm_sample_id=sample.name,
                submitter=self.gisaid_submitter,
                fn=f"{family_id}.fasta",
                covv_collection_date=self.lims_api.get_sample_attribute(
                    lims_id=sample_id, key="collection_date"
                ),
                region=self.lims_api.get_sample_attribute(lims_id=sample_id, key="region"),
                region_code=self.lims_api.get_sample_attribute(
                    lims_id=sample_id, key="region_code"
                ),
                covv_orig_lab=self.lims_api.get_sample_attribute(
                    lims_id=sample_id, key="original_lab"
                ),
                covv_orig_lab_addr=self.lims_api.get_sample_attribute(
                    lims_id=sample_id, key="original_lab_address"
                ),
            )
            gisaid_samples.append(gisaid_sample)
        return gisaid_samples

    def upload(self, csv_file: Path, fasta_file: Path) -> None:
        """Load batch data to GISAID using the gisiad cli."""

        load_call: list = [
            "CoV",
            "upload",
            "--csv",
            str(csv_file.absolute()),
            "--fasta",
            str(fasta_file.absolute()),
        ]
        self.process.run_command(parameters=load_call)

        if self.process.stderr:
            LOG.info(f"gisaid stderr:\n{self.process.stderr}")
        if self.process.stdout:
            LOG.info(f"gisaid stdout:\n{self.process.stdout}")

    def __str__(self):
        return f"GisaidAPI(dry_run: {self.dry_run})"



import logging
from typing import List

from cg.apps.demultiplex.sample_sheet.novaseq_sample_sheet import SampleSheetCreator
from cg.apps.lims.samplesheet import LimsFlowcellSample
from cg.exc import FlowcellError
from cg.models.demultiplex.run_parameters import RunParameters
from cg.store.models import Flowcell

LOG = logging.getLogger(__name__)


def create_sample_sheet(flowcell: Flowcell, lims_samples: List[LimsFlowcellSample]) -> str:
    """Create a sample sheet for a flowcell"""
    if flowcell.sample_sheet_path.exists():
        message = f"Sample sheet {flowcell.sample_sheet_path} already exists!"
        LOG.warning(message)
        raise FileExistsError(message)

    run_parameters: RunParameters = flowcell.run_parameters_object

    if run_parameters.flowcell_type != "novaseq":
        message = f"Can only demultiplex novaseq with cg. Found type {run_parameters.flowcell_type}"
        LOG.warning(message)
        raise FlowcellError(message=message)

    sample_sheet_creator = SampleSheetCreator(
        flowcell_id=flowcell.flowcell_id,
        lims_samples=lims_samples,
        run_parameters=run_parameters,
    )
    return sample_sheet_creator.construct_sample_sheet()



"""Functionality to export data from the database"""
import logging

import click
from cg.models.cg_config import CGConfig
from cg.store import Store

LOG = logging.getLogger(__name__)


@click.group(invoke_without_command=True)
@click.argument("table")
@click.argument("identifier")
@click.pass_obj
def export(context: CGConfig, table: str, identifier: str):
    """Get information about almost anything in the store."""
    status_db: Store = context.status_db
    db_func = getattr(status_db, f"{table}")
    db_obj = db_func(identifier)

    if db_obj is None:
        LOG.error("%s: %s not found", identifier, table)
        raise click.Abort

    click.echo(db_obj.to_dict())



"""CLI for delivering files with CG"""
import logging
from pathlib import Path
from typing import List, Optional

import click
from cg.constants.delivery import PIPELINE_ANALYSIS_OPTIONS, PIPELINE_ANALYSIS_TAG_MAP
from cg.meta.deliver import DeliverAPI
from cg.meta.deliver_ticket import DeliverTicketAPI
from cg.meta.rsync import RsyncAPI
from cg.models.cg_config import CGConfig
from cg.store import Store, models

LOG = logging.getLogger(__name__)


@click.group()
def deliver():
    """Deliver files with CG."""
    LOG.info("Running CG deliver")


@deliver.command(name="analysis")
@click.option("-c", "--case-id", help="Deliver the files for a specific case")
@click.option(
    "-t", "--ticket-id", type=int, help="Deliver the files for ALL cases connected to a ticket"
)
@click.option("-d", "--delivery-type", type=click.Choice(PIPELINE_ANALYSIS_OPTIONS), required=True)
@click.option("--dry-run", is_flag=True)
@click.pass_obj
def deliver_analysis(
    context: CGConfig,
    case_id: Optional[str],
    ticket_id: Optional[int],
    delivery_type: str,
    dry_run: bool,
):
    """Deliver analysis files to customer inbox

    Files can be delivered either on case level or for all cases connected to a ticket.
    Any of those needs to be specified.
    """
    if not (case_id or ticket_id):
        LOG.info("Please provide a case-id or ticket-id")
        return

    inbox: str = context.delivery_path
    if not inbox:
        LOG.info("Please specify the root path for where files should be delivered")
        return

    status_db: Store = context.status_db
    deliver_api = DeliverAPI(
        store=status_db,
        hk_api=context.housekeeper_api,
        case_tags=PIPELINE_ANALYSIS_TAG_MAP[delivery_type]["case_tags"],
        sample_tags=PIPELINE_ANALYSIS_TAG_MAP[delivery_type]["sample_tags"],
        project_base_path=Path(inbox),
        delivery_type=delivery_type,
    )
    deliver_api.set_dry_run(dry_run)
    cases: List[models.Family] = []
    if case_id:
        case_obj: models.Family = status_db.family(case_id)
        if not case_obj:
            LOG.warning("Could not find case %s", case_id)
            return
        cases.append(case_obj)
    else:
        cases = status_db.get_cases_from_ticket(ticket_id=ticket_id).all()
        if not cases:
            LOG.warning("Could not find cases for ticket_id %s", ticket_id)
            return

    for case_obj in cases:
        deliver_api.deliver_files(case_obj=case_obj)


@deliver.command(name="rsync")
@click.argument("ticket_id", type=int, required=True)
@click.option("--dry-run", is_flag=True)
@click.pass_obj
def rsync(context: CGConfig, ticket_id: int, dry_run: bool):
    """The folder generated using the "cg deliver analysis" command will be
    rsynced with this function to the customers inbox on caesar.
    """
    rsync_api = RsyncAPI(config=context)
    rsync_api.run_rsync_command(ticket_id=ticket_id, dry_run=dry_run)


@deliver.command(name="concatenate")
@click.argument("ticket_id", type=int, required=True)
@click.option("--dry-run", is_flag=True)
@click.pass_context
def concatenate(context: click.Context, ticket_id: int, dry_run: bool):
    """The fastq files in the folder generated using "cg deliver analysis"
    will be concatenated into one forward and one reverse fastq file.
    """
    cg_context: CGConfig = context.obj
    deliver_ticket_api = DeliverTicketAPI(config=cg_context)
    deliver_ticket_api.concatenate(ticket_id=ticket_id, dry_run=dry_run)


@deliver.command(name="ticket")
@click.option(
    "-t",
    "--ticket-id",
    type=int,
    help="Deliver and rsync the files for ALL cases connected to a ticket",
    required=True,
)
@click.option("-d", "--delivery-type", type=click.Choice(PIPELINE_ANALYSIS_OPTIONS), required=True)
@click.option("--dry-run", is_flag=True)
@click.pass_context
def deliver_ticket(
    context: click.Context,
    ticket_id: int,
    delivery_type: str,
    dry_run: bool,
):
    """Will first collect hard links in the customer inbox then
    concatenate fastq files if needed and finally send the folder
    from customer inbox hasta to the customer inbox on caesar.
    """
    cg_context: CGConfig = context.obj
    deliver_ticket_api = DeliverTicketAPI(config=cg_context)
    is_upload_needed = deliver_ticket_api.check_if_upload_is_needed(ticket_id=ticket_id)
    if is_upload_needed:
        LOG.info("Delivering files to customer inbox on hasta")
        context.invoke(
            deliver_analysis, ticket_id=ticket_id, delivery_type=delivery_type, dry_run=dry_run
        )
    else:
        LOG.info("Files already delivered to customer inbox on hasta")
        return
    is_concatenation_needed = deliver_ticket_api.check_if_concatenation_is_needed(
        ticket_id=ticket_id
    )
    if is_concatenation_needed and delivery_type == "fastq":
        context.invoke(concatenate, ticket_id=ticket_id, dry_run=dry_run)
    context.invoke(rsync, ticket_id=ticket_id, dry_run=dry_run)



"""Handles presentation of different data types"""
from datetime import datetime


class Presenter:
    """Contains methods for presentation of different data types"""

    DEFAULT_NA = "N/A"
    DEFAULT_PRECISION = "N/A"

    def __init__(self, precision: int = DEFAULT_PRECISION, default_na: str = DEFAULT_NA):
        self._precision = precision
        self._default_na = default_na

    def process_float_string(self, raw_float_str: str, precision: int = 2) -> str:
        """Make a float value presentable"""

        return str(round(float(raw_float_str), precision)) if raw_float_str else self._default_na

    def process_float(self, raw_float: float, precision: int = 2) -> str:
        """Make a float value presentable"""

        return str(round(raw_float, precision)) if raw_float else self._default_na

    def process_string(self, raw_string: str) -> str:
        """Make a string value presentable"""

        return raw_string if raw_string else self._default_na

    def process_datetime(self, raw_date: datetime) -> str:
        """Make an date value presentable"""

        return str(raw_date.date()) if raw_date else self._default_na

    def process_int(self, raw_int: int) -> str:
        """Make an integer value presentable"""

        return str(raw_int) if raw_int else self._default_na

    def process_set(self, raw_set: set) -> str:
        """Make a set presentable"""

        return ", ".join(str(s) for s in raw_set) if raw_set else self._default_na

    def process_list(self, raw_list: list):
        """Make a list presentable"""

        if raw_list and isinstance(raw_list[0], str):
            return ", ".join(str(s) for s in raw_list) if raw_list else self._default_na

        return [self.process_obj(o) for o in raw_list]

    def process_dict(self, raw_dict: dict):
        """Make a dict presentable"""

        presentable_dict = {}
        for key, value in raw_dict.items():
            presentable_dict[key] = self.process_obj(value)

        return presentable_dict

    def process_obj(self, obj):
        """Make supported types presentable"""

        if obj is None:
            presentable_value = self._default_na
        elif isinstance(obj, str):
            if self._is_float(obj):
                presentable_value = self.process_float_string(obj)
            else:
                presentable_value = self.process_string(obj)
        elif isinstance(obj, dict):
            presentable_value = self.process_dict(obj)
        elif isinstance(obj, set):
            presentable_value = self.process_set(obj)
        elif isinstance(obj, list):
            presentable_value = self.process_list(obj)
        elif isinstance(obj, datetime):
            presentable_value = self.process_datetime(obj)
        elif isinstance(obj, int):
            presentable_value = self.process_int(obj)
        elif isinstance(obj, float):
            presentable_value = self.process_float(obj)
        else:
            raise TypeError(f"Unsupported {obj} of type {type(obj)} in {self.__class__.__name__}")

        return presentable_value

    @staticmethod
    def _is_float(value):
        try:
            float(value)
            return True
        except ValueError:
            return False



"""Fixtures for store tests"""
import datetime as dt
from pathlib import Path

import pytest

# Files fixtures
from cg.constants import Pipeline
from cg.store import Store, models


@pytest.fixture
def application_versions_file(fixtures_dir: Path) -> str:
    """ "application version import file"""
    _file = fixtures_dir / "store/api/application_versions.xlsx"
    return str(_file)


@pytest.fixture
def applications_file(fixtures_dir):
    """ "application import file"""
    _file = fixtures_dir / "store/api/applications.xlsx"
    return str(_file)


@pytest.fixture(name="microbial_submitted_order")
def fixture_microbial_submitted_order():
    """Build an example order as it looks after submission to ."""

    def _get_item(name: str, internal_id: str, well_position: str, organism: str) -> dict:
        """Return a item"""
        ref_genomes = {
            "C. Jejuni": "NC_111",
            "M. upium": "NC_222",
            "C. difficile": "NC_333",
        }
        item = dict(
            name=name,
            internal_id=internal_id,
            reads="1000",
            container="96 well plate",
            container_name="hej",
            rml_plate_name=None,
            well_position=well_position,
            well_position_rml=None,
            sex=None,
            panels=None,
            require_qcok=True,
            application="MWRNXTR003",
            source=None,
            status=None,
            customer="cust015",
            family=None,
            priority="standard",
            capture_kit=None,
            comment="comment",
            index=None,
            reagent_label=None,
            tumour=False,
            custom_index=None,
            elution_buffer="Nuclease-free water",
            organism=organism,
            reference_genome=ref_genomes[organism],
            extraction_method="MagNaPure 96 (contact Clinical Genomics before " "submission)",
            analysis=str(Pipeline.FASTQ),
            concentration_sample="1",
            mother=None,
            father=None,
        )
        return item

    order = {
        "customer": "cust000",
        "name": "test order",
        "internal_id": "lims_reference",
        "comment": "test comment",
        "ticket_number": "123456",
        "items": [
            _get_item("Jag", "ms1", "D:5", "C. Jejuni"),
            _get_item("testar", "ms2", "H:5", "M. upium"),
            _get_item("list", "ms3", "A:6", "C. difficile"),
        ],
        "project_type": "microbial",
    }
    return order


@pytest.fixture(scope="function")
def microbial_store(base_store: Store, microbial_submitted_order):
    """Setup a store instance for testing analysis API."""
    customer = base_store.customer(microbial_submitted_order["customer"])

    for sample_data in microbial_submitted_order["items"]:
        application_version = base_store.application(sample_data["application"]).versions[0]
        organism = base_store.Organism(
            internal_id=sample_data["organism"], name=sample_data["organism"]
        )
        base_store.add(organism)
        sample = base_store.add_sample(
            comment=sample_data["comment"],
            internal_id=sample_data["internal_id"],
            name=sample_data["name"],
            priority=sample_data["priority"],
            reads=sample_data["reads"],
            reference_genome=sample_data["reference_genome"],
            sex="unknown",
            ticket=microbial_submitted_order["ticket_number"],
        )
        sample.application_version = application_version
        sample.customer = customer
        sample.organism = organism
        base_store.add(sample)

    base_store.commit()
    yield base_store


@pytest.fixture(scope="function", name="analysis_obj")
def fixture_analysis_obj(analysis_store: Store) -> models.Analysis:
    """Fetch a analysis object from a populated store"""
    return analysis_store.analyses()[0]


@pytest.fixture(scope="function")
def case_obj(analysis_obj: models.Analysis) -> models.Family:
    """Return a case models object."""
    return analysis_obj.family



"""Functions to get sample sheet information from Lims"""
import logging
import re
from typing import Iterable, List, Optional

from genologics.entities import Artifact, Container, Sample
from genologics.lims import Lims
from pydantic import BaseModel, Field

LOG = logging.getLogger(__name__)


class LimsFlowcellSample(BaseModel):
    flowcell_id: str = Field(..., alias="FCID")
    lane: int = Field(..., alias="Lane")
    sample_id: str = Field(..., alias="SampleID")
    sample_ref: str = Field("hg19", alias="SampleRef")
    index: str
    index2: str = ""
    description: str = ""
    sample_name: str = Field(..., alias="SampleName")
    control: str = Field("N", alias="Control")
    recipe: str = Field("R1", alias="Recipe")
    operator: str = Field("script", alias="Operator")
    project: str = Field(..., alias="Project")

    class Config:
        allow_population_by_field_name = True


def get_placement_lane(lane: str) -> int:
    """Parse out the lane information from an artifact.placement"""
    return int(lane.split(":")[0])


def get_non_pooled_artifacts(artifact: Artifact) -> List[Artifact]:
    """Find the parent artifact of the sample. Should hold the reagent_label"""
    artifacts = []

    if len(artifact.samples) == 1:
        artifacts.append(artifact)
        return artifacts

    for artifact_input in artifact.input_artifact_list():
        artifacts.extend(get_non_pooled_artifacts(artifact_input))

    return artifacts


def get_reagent_label(artifact) -> Optional[str]:
    """Get the first and only reagent label from an artifact"""
    labels: List[str] = artifact.reagent_labels
    if len(labels) > 1:
        raise ValueError("Expecting at most one reagent label. Got ({}).".format(len(labels)))
    return labels[0] if labels else None


def get_index(lims: Lims, label: str) -> str:
    """Parse out the sequence from a reagent label"""

    reagent_types = lims.get_reagent_types(name=label)

    if len(reagent_types) > 1:
        raise ValueError("Expecting at most one reagent type. Got ({}).".format(len(reagent_types)))

    try:
        reagent_type = reagent_types.pop()
    except IndexError:
        return ""
    sequence = reagent_type.sequence

    match = re.match(r"^.+ \((.+)\)$", label)
    if match:
        assert match.group(1) == sequence

    return sequence


def flowcell_samples(lims: Lims, flowcell_id: str) -> Iterable[LimsFlowcellSample]:
    LOG.info("Fetching samples from lims for flowcell %s", flowcell_id)
    containers: List[Container] = lims.get_containers(name=flowcell_id)
    if not containers:
        return []
    container: Container = containers[-1]  # only take the last one. See ÖA#217.
    raw_lanes: List[str] = sorted(container.placements.keys())
    for raw_lane in raw_lanes:
        lane: int = get_placement_lane(raw_lane)
        placement_artifact: Artifact = container.placements[raw_lane]
        non_pooled_artifacts: List[Artifact] = get_non_pooled_artifacts(placement_artifact)
        for artifact in non_pooled_artifacts:
            sample: Sample = artifact.samples[0]  # we are assured it only has one sample
            label: Optional[str] = get_reagent_label(artifact)
            index = get_index(lims=lims, label=label)
            yield LimsFlowcellSample(
                flowcell_id=flowcell_id,
                lane=lane,
                sample_id=sample.id,
                index=index,
                sample_name=sample.name,
                project=sample.project.name,
            )



import logging
from typing import Iterable, Optional

import click
from cg.models.cg_config import CGConfig
from cg.store import Store, models

LOG = logging.getLogger(__name__)


@click.group("reset")
def reset_cmd():
    """Reset information in the database."""
    pass


@reset_cmd.command()
@click.option("-c", "--case_id", help="internal case id, leave empty to process all")
@click.pass_obj
def observations(context: CGConfig, case_id: Optional[str]):
    """Reset observation links from an analysis to LoqusDB."""
    status_db: Store = context.status_db
    observations_uploaded: Iterable[models.Family]
    if case_id:
        observations_uploaded = [status_db.family(case_id)]
    else:
        observations_uploaded = status_db.observations_uploaded()

    for case_obj in observations_uploaded:
        LOG.info("This would reset observation links for: %s", case_obj.internal_id)

    click.confirm("Do you want to continue?", abort=True)

    for case_obj in observations_uploaded:
        status_db.reset_observations(case_obj.internal_id)
        LOG.info("Reset loqus observations for: %s", case_obj.internal_id)

    status_db.commit()
